@@ -245,6 +245,7 @@ size_t grib_iarray_used_size(grib_iarray *v);
 int grib_iarray_is_constant(grib_iarray *v);
 
 /* grib_vdarray.c */
+void grib_vdarray_print(const char* title, const grib_vdarray* vdarray);
 grib_vdarray *grib_vdarray_new(grib_context *c, size_t size, size_t incsize);
 grib_vdarray *grib_vdarray_resize(grib_context *c, grib_vdarray *v);
 grib_vdarray *grib_vdarray_push(grib_context *c, grib_vdarray *v, grib_darray *val);
@@ -806,7 +807,9 @@ long accessor_raw_get_offset(grib_accessor *a);
 /* grib_accessor_class_long_vector.c */
 
 /* grib_gaussian_reduced.c */
+void grib_get_reduced_row_wrapper(grib_handle *h, long pl, double lon_first, double lon_last, long *npoints, long *ilon_first, long *ilon_last);
 void grib_get_reduced_row(long pl, double lon_first, double lon_last, long *npoints, long *ilon_first, long *ilon_last);
+void grib_get_reduced_row2(long pl, double lon_first, double lon_last, long *npoints, long *ilon_first, long *ilon_last);
 
 /* grib_accessor_class_abstract_vector.c */
 
@@ -1459,6 +1462,8 @@ int is_productDefinitionTemplateNumber_Aerosol(long productDefinitionTemplateNum
 int is_index_file(const char *filename);
 char get_dir_separator_char(void);
 char *codes_getenv(const char *name);
+int expandedBoundingBox(grib_handle* h);
+size_t sum_of_pl_array(const long* pl, size_t plsize);
 
 /* bufr_util.c */
 int compute_bufr_key_rank(grib_handle *h, grib_string_list *keys, const char *key);
@@ -1470,6 +1475,8 @@ int strcmp_nocase(const char *s1, const char *s2);
 void rtrim(char *s);
 const char *extract_filename(const char *filepath);
 char **string_split(char *inputString, const char *delimiter);
+int string_to_long(const char* input, long* output);
+int string_ends_with(const char* str1, const char* str2);
 
 /* functions.c */
 long grib_op_eq(long a, long b);
