@@ -336,6 +336,7 @@ static grib_context default_grib_context = {
         0,                            /* ieee_packing               */
         0,                            /* bufrdc_mode                */
         0,                            /* bufr_set_to_missing_if_out_of_range */
+        0,                            /* bufr_quality_without_bitmap*/
         0,                            /* log_stream                 */
         0,                            /* classes                    */
         0,                            /* lists                      */
@@ -373,11 +374,13 @@ grib_context* grib_context_get_default()
         const char* keep_matrix = NULL;
         const char* bufrdc_mode = NULL;
         const char* bufr_set_to_missing_if_out_of_range = NULL;
+        const char* bufr_quality_without_bitmap = NULL;
         const char* file_pool_max_opened_files = NULL;
 
         write_on_fail = codes_getenv("ECCODES_GRIB_WRITE_ON_FAIL");
         bufrdc_mode = codes_getenv("ECCODES_BUFRDC_MODE_ON");
         bufr_set_to_missing_if_out_of_range = codes_getenv("ECCODES_BUFR_SET_TO_MISSING_IF_OUT_OF_RANGE");
+        bufr_quality_without_bitmap = codes_getenv("ECCODES_BUFR_QUALITY_WITHOUT_BITMAP");
         large_constant_fields = codes_getenv("ECCODES_GRIB_LARGE_CONSTANT_FIELDS");
         no_abort = codes_getenv("ECCODES_NO_ABORT");
         debug = codes_getenv("ECCODES_DEBUG");
@@ -474,6 +477,8 @@ grib_context* grib_context_get_default()
         default_grib_context.bufrdc_mode = bufrdc_mode ? atoi(bufrdc_mode) : 0;
         default_grib_context.bufr_set_to_missing_if_out_of_range = bufr_set_to_missing_if_out_of_range ?
                 atoi(bufr_set_to_missing_if_out_of_range) : 0;
+        default_grib_context.bufr_quality_without_bitmap = bufr_quality_without_bitmap ?
+                atoi(bufr_quality_without_bitmap) : 0;
         default_grib_context.file_pool_max_opened_files = file_pool_max_opened_files ?
                 atoi(file_pool_max_opened_files) : DEFAULT_FILE_POOL_MAX_OPENED_FILES;
     }
