@@ -146,7 +146,7 @@ static int destroy(grib_dumper* d)
 
 static char* lval_to_string(grib_context* c, long v)
 {
-    char* sval=grib_context_malloc_clear(c,sizeof(char)*40);
+    char* sval=(char*)grib_context_malloc_clear(c,sizeof(char)*40);
     if (v == GRIB_MISSING_LONG) sprintf(sval,"CODES_MISSING_LONG");
     else                        sprintf(sval,"%ld",v);
     return sval;
@@ -668,6 +668,7 @@ static void _dump_long_array(grib_handle* h, FILE* f, const char* key, const cha
     int cols=9,icount=0;
 
     if (grib_get_size(h,key,&size)==GRIB_NOT_FOUND) return;
+    if (size==0) return;
 
     fprintf(f,"    ivalues = (");
 
@@ -701,6 +702,7 @@ static void dump_section(grib_dumper* d, grib_accessor* a, grib_block_of_accesso
         _dump_long_array(h,self->dumper.out,"delayedDescriptorReplicationFactor","inputDelayedDescriptorReplicationFactor");
         _dump_long_array(h,self->dumper.out,"shortDelayedDescriptorReplicationFactor","inputShortDelayedDescriptorReplicationFactor");
         _dump_long_array(h,self->dumper.out,"extendedDelayedDescriptorReplicationFactor","inputExtendedDelayedDescriptorReplicationFactor");
+        _dump_long_array(h,self->dumper.out,"inputOverriddenReferenceValues","inputOverriddenReferenceValues");
         grib_dump_accessors_block(d,block);
         depth-=2;
     } else if (!grib_inline_strcmp(a->name,"groupNumber")) {
