@@ -52,6 +52,15 @@ int error=0;
 int count=0;
 int lastPrint=0;
 int force=0;
+
+/* ECC-651: Boolean 'two_way' set to 1 when '-2' option used */
+int two_way=0;
+/* Boolean 'handles_swapped' relevant in 'two_way' mode:
+ *  0 means: h1 is first file,  h2 is second file
+ *  1 means: h1 is second file, h2 is first file
+ */
+int handles_swapped=0;
+
 double maxAbsoluteError = 1e-19;
 int onlyListed=1;
 int headerMode=0;
@@ -118,7 +127,8 @@ grib_option grib_options[]={
     /*  {id, args, help}, on, command_line, value*/
     {"r",0,"Compare files in which the messages are not in the same order. This option is time expensive.\n",0,1,0},
     {"b:",0,0,0,1,0},
-    {"e",0,"Edition independent compare. It is used to compare grib edition 1 and 2.\n",0,1,0},
+    {"e",0,"Edition independent compare. It is used to compare GRIB edition 1 and 2.\n",0,1,0},
+    {"2",0,"Enable two-way comparison.\n",0,1,0},
     {"c:",0,0,0,1,0},
     {"S:","start","First field to be processed.\n",0,1,0},
     {"E:","end","Last field to be processed.\n",0,1,0},
@@ -183,6 +193,9 @@ int grib_tool_init(grib_runtime_options* options)
     if (grib_options_on("f")) force=1;
     else force=0;
 
+    if (grib_options_on("2")) two_way=1;
+    else two_way=0;
+
     verbose = grib_options_on("v");
 
     listFromCommandLine=0;
@@ -431,7 +444,23 @@ int grib_tool_new_handle_action(grib_runtime_options* options, grib_handle* h)
 
     if(compare_handles(global_handle,h,options)) {
         error++;
-        if (!force) exit(1);
+        if (!two_way) {
+            /* If two_way mode: Don't exit yet. Show further differences */
+            if (!force) exit(1);
+        }
+    }
+    if (two_way) {
+        /* ECC-651 and ECC-431 */
+        handles_swapped = 1;
+        if(compare_handles(h, global_handle, options)) {
+            error++;
+            if (!force) exit(1);
+        } else {
+            if (error) {
+                /* Error from first pass */
+                if (!force) exit(1);
+            }
+        }
     }
 
     grib_handle_delete(global_handle);
@@ -562,6 +591,8 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     double packingError1=0,packingError2=0;
     double value_tolerance=0;
     grib_context* c=h1->context;
+    const char* first_str = (handles_swapped==0? "1st" : "2nd");
+    const char* second_str = (handles_swapped==0? "2nd" : "1st");
 
     type1=type;
     type2=type;
@@ -573,7 +604,7 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     if( type1==GRIB_TYPE_UNDEFINED && (err = grib_get_native_type(h1,name,&type1)) != GRIB_SUCCESS)
     {
         printInfo(h1);
-        printf("Oops... cannot get type of [%s] in 1st field: %s\n",name,grib_get_error_message(err));
+        printf("Oops... cannot get type of [%s] in %s field: %s\n",name,first_str,grib_get_error_message(err));
         save_error(c,name);
         return err;
     }
@@ -583,24 +614,22 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         if(err == GRIB_NOT_FOUND)
         {
             printInfo(h1);
-            printf("[%s] not found in 2nd field\n",name);
+            printf("[%s] not found in %s field\n",name,second_str);
             save_error(c,name);
             return err;
         }
         printInfo(h1);
-        printf("Oops... cannot get type of [%s] in 2nd field: %s\n",name,grib_get_error_message(err));
+        printf("Oops... cannot get type of [%s] in %s field: %s\n",name,second_str,grib_get_error_message(err));
         save_error(c,name);
         return err;
     }
 
-    /*
-  if(type1 != type2)
-  {
-    printInfo(h1);
-    printf("Warning, [%s] has different types: 1st field: [%s], 2nd field: [%s]\n",
-        name,grib_get_type_name(type1),grib_get_type_name(type2));
-    return GRIB_TYPE_MISMATCH; 
-  }
+    /*  if(type1 != type2) {
+       printInfo(h1);
+       printf("Warning, [%s] has different types: 1st field: [%s], 2nd field: [%s]\n",
+            name,grib_get_type_name(type1),grib_get_type_name(type2));
+       return GRIB_TYPE_MISMATCH; 
+     }
      */
 
     if(type1 == GRIB_TYPE_LABEL)
@@ -613,7 +642,7 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     if((err = grib_get_size(h1,name,&len1)) != GRIB_SUCCESS)
     {
         printInfo(h1);
-        printf("Oops... cannot get size of [%s] in 1st field: %s\n",name,grib_get_error_message(err));
+        printf("Oops... cannot get size of [%s] in %s field: %s\n",name,first_str,grib_get_error_message(err));
         save_error(c,name);
         return err;
     }
@@ -623,25 +652,28 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         if(err == GRIB_NOT_FOUND)
         {
             printInfo(h1);
-            printf("[%s] not found in 2nd field\n",name);
+            printf("[%s] not found in %s field\n",name,second_str);
             save_error(c,name);
             return err;
         }
 
         printInfo(h1);
-        printf("Oops... cannot get size of [%s] in 2nd field: %s\n",name,grib_get_error_message(err));
+        printf("Oops... cannot get size of [%s] in %s field: %s\n",name,second_str,grib_get_error_message(err));
         save_error(c,name);
         return err;
     }
 
-    /*
-  if(len1 != len2 && type1 != GRIB_TYPE_STRING)
-  {
-    printInfo(h1);
-    printf("[%s] has different size: 1st field: %ld, 2nd field: %ld\n",name,(long)len1,(long)len2);
-    save_error(c,name);
-    return GRIB_COUNT_MISMATCH;
-  }
+    if (handles_swapped) {
+        /* Comparing a second time with handles swapped. Do not compare keys common to both handles */
+        return GRIB_SUCCESS;
+    }
+
+    /*  if(len1 != len2 && type1 != GRIB_TYPE_STRING)  {
+        printInfo(h1);
+        printf("[%s] has different size: 1st field: %ld, 2nd field: %ld\n",name,(long)len1,(long)len2);
+        save_error(c,name);
+        return GRIB_COUNT_MISMATCH;
+    }
      */
 
     isMissing1= ( (grib_is_missing(h1,name,&err1)==1) && (err1 == 0) ) ? 1 : 0;
@@ -653,18 +685,18 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
     }
 
     if (isMissing1==1) {
-        if (verbose) printf(" is set to missing in 1st field\n");
+        if (verbose) printf(" is set to missing in %s field\n",first_str);
         printInfo(h1);
-        printf("%s is set to missing in 1st field is not missing in 2nd field\n",name);
+        printf("%s is set to missing in %s field is not missing in %s field\n",name,first_str,second_str);
         err1 = GRIB_VALUE_MISMATCH;
         save_error(c,name);
         return GRIB_VALUE_MISMATCH;
     }
 
     if (isMissing2==1) {
-        if (verbose) printf(" is set to missing in 1st field\n");
+        if (verbose) printf(" is set to missing in %s field\n",first_str);
         printInfo(h1);
-        printf("%s is set to missing in 2nd field is not missing in 1st field\n",name);
+        printf("%s is set to missing in %s field is not missing in %s field\n",name,second_str,first_str);
         err1 = GRIB_VALUE_MISMATCH;
         save_error(c,name);
         return GRIB_VALUE_MISMATCH;
@@ -682,16 +714,16 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         if((err1 = grib_get_string(h1,name,sval1,&len1)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get string value of [%s] in 1st field: %s\n",
-                    name,grib_get_error_message(err1));
+            printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    name,first_str,grib_get_error_message(err1));
             save_error(c,name);
         }
 
         if((err2 = grib_get_string(h2,name,sval2,&len2)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get string value of [%s] in 2nd field: %s\n",
-                    name,grib_get_error_message(err2));
+            printf("Oops... cannot get string value of [%s] in %s field: %s\n",
+                    name,second_str,grib_get_error_message(err2));
             save_error(c,name);
         }
 
@@ -724,16 +756,16 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         if((err1 = grib_get_long_array(h1,name,lval1,&len1)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get long value of [%s] in 1st field: %s\n",
-                    name,grib_get_error_message(err1));
+            printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                    name,first_str,grib_get_error_message(err1));
             save_error(c,name);
         }
 
         if((err2 = grib_get_long_array(h2,name,lval2,&len2)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get long value of [%s] in 2nd field: %s\n",
-                    name,grib_get_error_message(err2));
+            printf("Oops... cannot get long value of [%s] in %s field: %s\n",
+                    name,second_str,grib_get_error_message(err2));
             save_error(c,name);
         }
 
@@ -837,16 +869,16 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         if((err1 = grib_get_double_array(h1,name,dval1,&len1)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get double value of [%s] in 1st field: %s\n",
-                    name,grib_get_error_message(err1));
+            printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                    name,first_str,grib_get_error_message(err1));
             save_error(c,name);
         }
 
         if((err2 = grib_get_double_array(h2,name,dval2,&len2)) != GRIB_SUCCESS)
         {
             printInfo(h1);
-            printf("Oops... cannot get double value of [%s] in 2nd field: %s\n",
-                    name,grib_get_error_message(err2));
+            printf("Oops... cannot get double value of [%s] in %s field: %s\n",
+                    name,second_str,grib_get_error_message(err2));
             save_error(c,name);
         }
 
@@ -945,16 +977,16 @@ static int compare_values(grib_runtime_options* options,grib_handle* h1,grib_han
         {
             printInfo(h1);
             save_error(c,name);
-            printf("Oops... cannot get bytes value of [%s] in 1st field: %s\n",
-                    name,grib_get_error_message(err1));
+            printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
+                    name,first_str,grib_get_error_message(err1));
         }
 
         if((err2 = grib_get_bytes(h2,name,uval2,&len2)) != GRIB_SUCCESS)
         {
             printInfo(h1);
             save_error(c,name);
-            printf("Oops... cannot get bytes value of [%s] in 2nd field: %s\n",
-                    name,grib_get_error_message(err2));
+            printf("Oops... cannot get bytes value of [%s] in %s field: %s\n",
+                    name,second_str,grib_get_error_message(err2));
         }
 
         if(err1 == GRIB_SUCCESS && err2 == GRIB_SUCCESS)
